import 'babel-polyfill';
import commander from 'commander';
import { exists, readFile, readdir, stat, unlink, writeFile } from 'mz/fs';
import { exec } from 'mz/child_process';
import path, { join, resolve } from 'path';
import readline from 'mz/readline';
import requireUncached from 'require-uncached';
import { spawn } from 'child_process';
import { copy, move, readFile as readFile$1, unlink as unlink$1, writeFile as writeFile$1 } from 'fs-promise';
import git from 'simple-git/promise';
import zlib from 'zlib';
import opn from 'opn';

function makeCLIFn(commandByPath) {
  return function _callee(path$$1) {
    return regeneratorRuntime.async(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            _context.next = 3;
            return regeneratorRuntime.awrap(exec(commandByPath(path$$1), { maxBuffer: 5 * 1024 * 1024 }));

          case 3:
            return _context.abrupt('return', { path: path$$1, error: null });

          case 6:
            _context.prev = 6;
            _context.t0 = _context['catch'](0);
            return _context.abrupt('return', { path: path$$1, error: _context.t0.message });

          case 9:
          case 'end':
            return _context.stop();
        }
      }
    }, null, this, [[0, 6]]);
  };
}

/**
 * Run the given one-argument async function on an array of arguments, keeping a
 * logical worker pool to increase throughput without overloading the system.
 *
 * Results are provided as they come in with the result handler. Results look
 * like {index: 3, result: "Hello"}. This can be used e.g. to update a progress
 * bar.
 *
 * An array of all results is returned at the end.
 */
var runInParallel$1 = (function runInParallel(args, asyncFn, numConcurrentProcesses, resultHandler) {
  var _this = this;

  var results, activePromises, handleResult, _loop, i;

  return regeneratorRuntime.async(function runInParallel$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          results = [];
          activePromises = {};

          handleResult = function handleResult(_ref) {
            var index = _ref.index,
                result = _ref.result;

            results[index] = result;
            delete activePromises[index];
            resultHandler({ index: index, result: result });
          };

          _loop = function _callee2(i) {
            var arg;
            return regeneratorRuntime.async(function _callee2$(_context2) {
              while (1) {
                switch (_context2.prev = _context2.next) {
                  case 0:
                    arg = args[i];

                    activePromises[i] = function _callee() {
                      return regeneratorRuntime.async(function _callee$(_context) {
                        while (1) {
                          switch (_context.prev = _context.next) {
                            case 0:
                              _context.t0 = i;
                              _context.next = 3;
                              return regeneratorRuntime.awrap(asyncFn(arg));

                            case 3:
                              _context.t1 = _context.sent;
                              return _context.abrupt("return", {
                                index: _context.t0,
                                result: _context.t1
                              });

                            case 5:
                            case "end":
                              return _context.stop();
                          }
                        }
                      }, null, this);
                    }();

                    if (!(Object.keys(activePromises).length >= numConcurrentProcesses)) {
                      _context2.next = 8;
                      break;
                    }

                    _context2.t0 = handleResult;
                    _context2.next = 6;
                    return regeneratorRuntime.awrap(Promise.race(Object.values(activePromises)));

                  case 6:
                    _context2.t1 = _context2.sent;
                    (0, _context2.t0)(_context2.t1);

                  case 8:
                  case "end":
                    return _context2.stop();
                }
              }
            }, null, _this);
          };

          i = 0;

        case 5:
          if (!(i < args.length)) {
            _context3.next = 11;
            break;
          }

          _context3.next = 8;
          return regeneratorRuntime.awrap(_loop(i));

        case 8:
          i++;
          _context3.next = 5;
          break;

        case 11:
          if (!(Object.keys(activePromises).length > 0)) {
            _context3.next = 19;
            break;
          }

          _context3.t0 = handleResult;
          _context3.next = 15;
          return regeneratorRuntime.awrap(Promise.race(Object.values(activePromises)));

        case 15:
          _context3.t1 = _context3.sent;
          (0, _context3.t0)(_context3.t1);
          _context3.next = 11;
          break;

        case 19:
          return _context3.abrupt("return", results);

        case 20:
        case "end":
          return _context3.stop();
      }
    }
  }, null, this);
});

function pluralize(num, noun) {
  return num === 1 ? num + " " + noun : num + " " + noun + "s";
}

var NUM_CONCURRENT_PROCESSES = 8;

/**
 * Run the given command in parallel, showing a progress bar of results.
 *
 * The provided async function should return an object that at least contains
 * a field called "error" that is truthy if there was a problem, but may contain
 * any other fields.
 */
var runWithProgressBar$1 = (function runWithProgressBar(description, files, asyncFn) {
  var _ref = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {},
      runInSeries = _ref.runInSeries;

  var numProcessed, numFailures, numTotal, numConcurrentProcesses, results;
  return regeneratorRuntime.async(function runWithProgressBar$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          numProcessed = 0;
          numFailures = 0;
          numTotal = files.length;

          console.log(description);
          numConcurrentProcesses = runInSeries ? 1 : NUM_CONCURRENT_PROCESSES;
          _context.next = 7;
          return regeneratorRuntime.awrap(runInParallel$1(files, asyncFn, numConcurrentProcesses, function (_ref2) {
            var result = _ref2.result;

            if (result.error) {
              numFailures++;
            }
            numProcessed++;
            var errorString = numFailures === 0 ? '' : ' (' + pluralize(numFailures, 'failure') + ' so far)';
            process.stdout.write('\r' + numProcessed + '/' + numTotal + errorString);
          }));

        case 7:
          results = _context.sent;

          process.stdout.write('\n');
          return _context.abrupt('return', results);

        case 10:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this);
});

var check$1 = (function check(config) {
  var filesToProcess, _config$decaffeinateA, decaffeinateArgs, decaffeinatePath, decaffeinateResults;

  return regeneratorRuntime.async(function check$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          filesToProcess = config.filesToProcess, _config$decaffeinateA = config.decaffeinateArgs, decaffeinateArgs = _config$decaffeinateA === undefined ? [] : _config$decaffeinateA, decaffeinatePath = config.decaffeinatePath;
          _context.next = 3;
          return regeneratorRuntime.awrap(runWithProgressBar$1('Doing a dry run of decaffeinate on ' + pluralize(filesToProcess.length, 'file') + '...', filesToProcess, makeCLIFn(function (path$$1) {
            return decaffeinatePath + ' ' + decaffeinateArgs.join(' ') + ' < ' + path$$1;
          })));

        case 3:
          decaffeinateResults = _context.sent;
          _context.next = 6;
          return regeneratorRuntime.awrap(printResults(decaffeinateResults));

        case 6:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this);
});

function printResults(results) {
  var errorResults, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, result, successPaths;

  return regeneratorRuntime.async(function printResults$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          errorResults = results.filter(function (r) {
            return r.error !== null;
          });

          if (!(errorResults.length === 0)) {
            _context2.next = 6;
            break;
          }

          console.log('All checks succeeded! decaffeinate can convert all ' + pluralize(results.length, 'file') + '.');
          console.log('Run "bulk-decaffeinate convert" to convert the files to JavaScript.');
          _context2.next = 37;
          break;

        case 6:
          console.log(pluralize(errorResults.length, 'file') + ' failed to convert:');
          _iteratorNormalCompletion = true;
          _didIteratorError = false;
          _iteratorError = undefined;
          _context2.prev = 10;
          for (_iterator = errorResults[Symbol.iterator](); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            result = _step.value;

            console.log(result.path);
          }
          _context2.next = 18;
          break;

        case 14:
          _context2.prev = 14;
          _context2.t0 = _context2['catch'](10);
          _didIteratorError = true;
          _iteratorError = _context2.t0;

        case 18:
          _context2.prev = 18;
          _context2.prev = 19;

          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }

        case 21:
          _context2.prev = 21;

          if (!_didIteratorError) {
            _context2.next = 24;
            break;
          }

          throw _iteratorError;

        case 24:
          return _context2.finish(21);

        case 25:
          return _context2.finish(18);

        case 26:
          successPaths = results.filter(function (r) {
            return r.error === null;
          }).map(function (r) {
            return r.path;
          });

          console.log();
          _context2.next = 30;
          return regeneratorRuntime.awrap(writeFile('decaffeinate-errors.log', getVerboseErrors(results)));

        case 30:
          _context2.next = 32;
          return regeneratorRuntime.awrap(writeFile('decaffeinate-results.json', JSON.stringify(results, null, 2)));

        case 32:
          _context2.next = 34;
          return regeneratorRuntime.awrap(writeFile('decaffeinate-successful-files.txt', successPaths.join('\n')));

        case 34:
          console.log('Wrote decaffeinate-errors.log and decaffeinate-results.json with more detailed info.');
          console.log('To open failures in the online repl, run "bulk-decaffeinate view-errors".');
          console.log('To convert the successful files, run "bulk-decaffeinate convert -p decaffeinate-successful-files.txt".');

        case 37:
        case 'end':
          return _context2.stop();
      }
    }
  }, null, this, [[10, 14, 18, 26], [19,, 21, 25]]);
}

function getVerboseErrors(results) {
  var errorMessages = [];
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = results[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var _step2$value = _step2.value,
          path$$1 = _step2$value.path,
          error = _step2$value.error;

      if (error) {
        errorMessages.push('===== ' + path$$1);
        errorMessages.push(getStdout(error));
      }
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  return errorMessages.join('\n');
}

function getStdout(message) {
  var matchString = '\nstdin: ';
  if (message.indexOf(matchString) !== -1) {
    return message.substring(message.indexOf(matchString) + matchString.length);
  } else {
    return message.substring(message.indexOf('\n') + 1);
  }
}

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();







var get = function get(object, property, receiver) {
  if (object === null) object = Function.prototype;
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent === null) {
      return undefined;
    } else {
      return get(parent, property, receiver);
    }
  } else if ("value" in desc) {
    return desc.value;
  } else {
    var getter = desc.get;

    if (getter === undefined) {
      return undefined;
    }

    return getter.call(receiver);
  }
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};











var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};



var set = function set(object, property, value, receiver) {
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent !== null) {
      set(parent, property, value, receiver);
    }
  } else if ("value" in desc && desc.writable) {
    desc.value = value;
  } else {
    var setter = desc.set;

    if (setter !== undefined) {
      setter.call(receiver, value);
    }
  }

  return value;
};















var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

/**
 * Recursively discover any matching files in the current directory, ignoring
 * things like node_modules and .git.
 */
var getFilesUnderPath$1 = (function getFilesUnderPath(dirPath, pathPredicate) {
  var resultFiles, children, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, child, childPath, subdirCoffeeFiles;

  return regeneratorRuntime.async(function getFilesUnderPath$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          resultFiles = [];
          _context.next = 3;
          return regeneratorRuntime.awrap(readdir(dirPath));

        case 3:
          children = _context.sent;
          _iteratorNormalCompletion = true;
          _didIteratorError = false;
          _iteratorError = undefined;
          _context.prev = 7;
          _iterator = children[Symbol.iterator]();

        case 9:
          if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
            _context.next = 27;
            break;
          }

          child = _step.value;

          if (!['node_modules', '.git'].includes(child)) {
            _context.next = 13;
            break;
          }

          return _context.abrupt('continue', 24);

        case 13:
          childPath = join(dirPath, child);
          _context.next = 16;
          return regeneratorRuntime.awrap(stat(childPath));

        case 16:
          if (!_context.sent.isDirectory()) {
            _context.next = 23;
            break;
          }

          _context.next = 19;
          return regeneratorRuntime.awrap(getFilesUnderPath(childPath, pathPredicate));

        case 19:
          subdirCoffeeFiles = _context.sent;

          resultFiles.push.apply(resultFiles, toConsumableArray(subdirCoffeeFiles));
          _context.next = 24;
          break;

        case 23:
          if (pathPredicate(child)) {
            resultFiles.push(childPath);
          }

        case 24:
          _iteratorNormalCompletion = true;
          _context.next = 9;
          break;

        case 27:
          _context.next = 33;
          break;

        case 29:
          _context.prev = 29;
          _context.t0 = _context['catch'](7);
          _didIteratorError = true;
          _iteratorError = _context.t0;

        case 33:
          _context.prev = 33;
          _context.prev = 34;

          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }

        case 36:
          _context.prev = 36;

          if (!_didIteratorError) {
            _context.next = 39;
            break;
          }

          throw _iteratorError;

        case 39:
          return _context.finish(36);

        case 40:
          return _context.finish(33);

        case 41:
          return _context.abrupt('return', resultFiles);

        case 42:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this, [[7, 29, 33, 41], [34,, 36, 40]]);
});

var clean$1 = (function clean() {
  var filesToDelete, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, path$$1;

  return regeneratorRuntime.async(function clean$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return regeneratorRuntime.awrap(getFilesUnderPath$1('.', function (p) {
            return p.endsWith('.original.coffee');
          }));

        case 2:
          filesToDelete = _context.sent;

          if (!(filesToDelete.length === 0)) {
            _context.next = 6;
            break;
          }

          console.log('No .original.coffee files were found.');
          return _context.abrupt('return');

        case 6:
          _iteratorNormalCompletion = true;
          _didIteratorError = false;
          _iteratorError = undefined;
          _context.prev = 9;
          _iterator = filesToDelete[Symbol.iterator]();

        case 11:
          if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
            _context.next = 19;
            break;
          }

          path$$1 = _step.value;

          console.log('Deleting ' + path$$1);
          _context.next = 16;
          return regeneratorRuntime.awrap(unlink(path$$1));

        case 16:
          _iteratorNormalCompletion = true;
          _context.next = 11;
          break;

        case 19:
          _context.next = 25;
          break;

        case 21:
          _context.prev = 21;
          _context.t0 = _context['catch'](9);
          _didIteratorError = true;
          _iteratorError = _context.t0;

        case 25:
          _context.prev = 25;
          _context.prev = 26;

          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }

        case 28:
          _context.prev = 28;

          if (!_didIteratorError) {
            _context.next = 31;
            break;
          }

          throw _iteratorError;

        case 31:
          return _context.finish(28);

        case 32:
          return _context.finish(25);

        case 33:
          console.log('Done deleting .original.coffee files.');

        case 34:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this, [[9, 21, 25, 33], [26,, 28, 32]]);
});

var PREFIX = 'bulk-decaffeinate CLIError: ';

/**
 * Exception class for a nice-looking error.
 *
 * Apparently async/await propagation doesn't preserve the exception, so to work
 * around this, we put a special prefix at the start of CLI errors and format
 * the error without a stack trace if the message starts with that prefix.
 */

var CLIError = function (_Error) {
  inherits(CLIError, _Error);

  function CLIError(message) {
    classCallCheck(this, CLIError);
    return possibleConstructorReturn(this, (CLIError.__proto__ || Object.getPrototypeOf(CLIError)).call(this, PREFIX + message));
  }

  createClass(CLIError, null, [{
    key: 'formatError',
    value: function formatError(e) {
      if (!e) {
        return e;
      }
      if (e.message.startsWith(PREFIX)) {
        return e.message.substring(PREFIX.length);
      } else {
        return e;
      }
    }
  }]);
  return CLIError;
}(Error);

/**
 * Read a list of .coffee files from a file and return it. Verify that all files
 * end in .coffee and that the files actually exist.
 */
var getCoffeeFilesFromPathFile$1 = (function getCoffeeFilesFromPathFile(filePath, requireValidFiles) {
  var fileContents, lines, resultLines, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, line;

  return regeneratorRuntime.async(function getCoffeeFilesFromPathFile$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return regeneratorRuntime.awrap(readFile(filePath));

        case 2:
          fileContents = _context.sent;
          lines = fileContents.toString().split('\n');
          resultLines = [];
          _iteratorNormalCompletion = true;
          _didIteratorError = false;
          _iteratorError = undefined;
          _context.prev = 8;
          _iterator = lines[Symbol.iterator]();

        case 10:
          if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
            _context.next = 33;
            break;
          }

          line = _step.value;

          line = line.trim();

          if (!(line.length === 0 || line.startsWith('#'))) {
            _context.next = 15;
            break;
          }

          return _context.abrupt('continue', 30);

        case 15:
          if (line.endsWith('.coffee')) {
            _context.next = 21;
            break;
          }

          if (!requireValidFiles) {
            _context.next = 20;
            break;
          }

          throw new CLIError('The line "' + line + '" must be a file path ending in .coffee.');

        case 20:
          return _context.abrupt('continue', 30);

        case 21:
          _context.next = 23;
          return regeneratorRuntime.awrap(exists(line));

        case 23:
          if (_context.sent) {
            _context.next = 29;
            break;
          }

          if (!requireValidFiles) {
            _context.next = 28;
            break;
          }

          throw new CLIError('The file "' + line + '" did not exist.');

        case 28:
          return _context.abrupt('continue', 30);

        case 29:
          resultLines.push(line);

        case 30:
          _iteratorNormalCompletion = true;
          _context.next = 10;
          break;

        case 33:
          _context.next = 39;
          break;

        case 35:
          _context.prev = 35;
          _context.t0 = _context['catch'](8);
          _didIteratorError = true;
          _iteratorError = _context.t0;

        case 39:
          _context.prev = 39;
          _context.prev = 40;

          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }

        case 42:
          _context.prev = 42;

          if (!_didIteratorError) {
            _context.next = 45;
            break;
          }

          throw _iteratorError;

        case 45:
          return _context.finish(42);

        case 46:
          return _context.finish(39);

        case 47:
          return _context.abrupt('return', resultLines);

        case 48:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this, [[8, 35, 39, 47], [40,, 42, 46]]);
});

/**
 * Recursively discover any .coffee files in the current directory, ignoring
 * things like node_modules and .git.
 */
var getCoffeeFilesUnderPath$1 = (function getCoffeeFilesUnderPath(dirPath) {
  return regeneratorRuntime.async(function getCoffeeFilesUnderPath$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return regeneratorRuntime.awrap(getFilesUnderPath$1(dirPath, function (p) {
            return p.endsWith('.coffee') && !p.endsWith('.original.coffee');
          }));

        case 2:
          return _context.abrupt('return', _context.sent);

        case 3:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this);
});

/**
 * Variant of exec that connects stdout, stderr, and stdin, mostly so console
 * output is shown continuously. As with the mz version of exec, this returns a
 * promise that resolves when the shell command finishes.
 */
function execLive(command) {
  return new Promise(function (resolve$$1, reject) {
    var childProcess = spawn('/bin/sh', ['-c', command], { stdio: 'inherit' });
    childProcess.on('close', function (code) {
      if (code === 0) {
        resolve$$1();
      } else {
        reject();
      }
    });
  });
}

/**
 * Resolve the configuration from a number of sources: any number of config
 * files and CLI options. Then "canonicalize" the config, e.g. by resolving the
 * list of files to process and inferring any paths if necessary.
 *
 * Some commands don't actually use the list of files, so we shouldn't do file
 * sanity checks for those.
 */
var resolveConfig$1 = (function resolveConfig(commander$$1) {
  var requireValidFiles = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

  var config, currentDirFiles, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, filename, filesToProcess;

  return regeneratorRuntime.async(function resolveConfig$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          config = {};
          _context.next = 3;
          return regeneratorRuntime.awrap(readdir('.'));

        case 3:
          currentDirFiles = _context.sent;

          currentDirFiles.sort();
          _iteratorNormalCompletion = true;
          _didIteratorError = false;
          _iteratorError = undefined;
          _context.prev = 8;
          _iterator = currentDirFiles[Symbol.iterator]();

        case 10:
          if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
            _context.next = 18;
            break;
          }

          filename = _step.value;
          _context.next = 14;
          return regeneratorRuntime.awrap(applyPossibleConfig(filename, config));

        case 14:
          config = _context.sent;

        case 15:
          _iteratorNormalCompletion = true;
          _context.next = 10;
          break;

        case 18:
          _context.next = 24;
          break;

        case 20:
          _context.prev = 20;
          _context.t0 = _context['catch'](8);
          _didIteratorError = true;
          _iteratorError = _context.t0;

        case 24:
          _context.prev = 24;
          _context.prev = 25;

          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }

        case 27:
          _context.prev = 27;

          if (!_didIteratorError) {
            _context.next = 30;
            break;
          }

          throw _iteratorError;

        case 30:
          return _context.finish(27);

        case 31:
          return _context.finish(24);

        case 32:
          config = getCLIParamsConfig(config, commander$$1);
          _context.next = 35;
          return regeneratorRuntime.awrap(resolveFilesToProcess(config, requireValidFiles));

        case 35:
          filesToProcess = _context.sent;

          filesToProcess = resolveFileFilter(filesToProcess, config);

          if (!requireValidFiles) {
            _context.next = 40;
            break;
          }

          _context.next = 40;
          return regeneratorRuntime.awrap(validateFilesToProcess(filesToProcess));

        case 40:
          _context.t1 = config.decaffeinateArgs;
          _context.t2 = filesToProcess;
          _context.t3 = config.fixImportsConfig;
          _context.t4 = config.jscodeshiftScripts;
          _context.t5 = config.landConfig;
          _context.t6 = config.mochaEnvFilePattern;
          _context.t7 = config.codePrefix;
          _context.t8 = config.landBase;
          _context.t9 = config.skipVerify;
          _context.next = 51;
          return regeneratorRuntime.awrap(resolveDecaffeinatePath(config));

        case 51:
          _context.t10 = _context.sent;
          _context.next = 54;
          return regeneratorRuntime.awrap(resolveJscodeshiftPath(config));

        case 54:
          _context.t11 = _context.sent;
          _context.next = 57;
          return regeneratorRuntime.awrap(resolveEslintPath(config));

        case 57:
          _context.t12 = _context.sent;
          return _context.abrupt('return', {
            decaffeinateArgs: _context.t1,
            filesToProcess: _context.t2,
            fixImportsConfig: _context.t3,
            jscodeshiftScripts: _context.t4,
            landConfig: _context.t5,
            mochaEnvFilePattern: _context.t6,
            codePrefix: _context.t7,
            landBase: _context.t8,
            skipVerify: _context.t9,
            decaffeinatePath: _context.t10,
            jscodeshiftPath: _context.t11,
            eslintPath: _context.t12
          });

        case 59:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this, [[8, 20, 24, 32], [25,, 27, 31]]);
});

function applyPossibleConfig(filename, config) {
  var filePath, newConfig, _newConfig;

  return regeneratorRuntime.async(function applyPossibleConfig$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.t0 = !filename.startsWith('bulk-decaffeinate');

          if (_context2.t0) {
            _context2.next = 5;
            break;
          }

          _context2.next = 4;
          return regeneratorRuntime.awrap(stat(filename));

        case 4:
          _context2.t0 = _context2.sent.isDirectory();

        case 5:
          if (!_context2.t0) {
            _context2.next = 7;
            break;
          }

          return _context2.abrupt('return', config);

        case 7:
          filePath = resolve(filename);

          if (!filename.endsWith('.json')) {
            _context2.next = 23;
            break;
          }

          _context2.prev = 9;
          _context2.t1 = JSON;
          _context2.next = 13;
          return regeneratorRuntime.awrap(readFile(filePath));

        case 13:
          _context2.t2 = _context2.sent;
          newConfig = _context2.t1.parse.call(_context2.t1, _context2.t2);
          return _context2.abrupt('return', Object.assign(config, newConfig));

        case 18:
          _context2.prev = 18;
          _context2.t3 = _context2['catch'](9);
          throw new CLIError('Error reading file ' + filePath + '. Make sure it is a valid JSON file.');

        case 21:
          _context2.next = 35;
          break;

        case 23:
          if (!filename.endsWith('.config.js')) {
            _context2.next = 34;
            break;
          }

          _context2.prev = 24;
          _newConfig = requireUncached(filePath);
          return _context2.abrupt('return', Object.assign(config, _newConfig));

        case 29:
          _context2.prev = 29;
          _context2.t4 = _context2['catch'](24);
          throw new CLIError('Error reading file ' + filePath + '. Make sure it is a valid JS file.');

        case 32:
          _context2.next = 35;
          break;

        case 34:
          return _context2.abrupt('return', config);

        case 35:
        case 'end':
          return _context2.stop();
      }
    }
  }, null, this, [[9, 18], [24, 29]]);
}

/**
 * Fill in a configuration from the CLI arguments.
 */
function getCLIParamsConfig(config, commander$$1) {
  var file = commander$$1.file,
      pathFile = commander$$1.pathFile,
      dir = commander$$1.dir,
      allowInvalidConstructors = commander$$1.allowInvalidConstructors,
      landBase = commander$$1.landBase,
      decaffeinatePath = commander$$1.decaffeinatePath,
      jscodeshiftPath = commander$$1.jscodeshiftPath,
      eslintPath = commander$$1.eslintPath;

  if (file && file.length > 0) {
    config.filesToProcess = file;
  }
  if (dir) {
    config.searchDirectory = dir;
  }
  if (pathFile) {
    config.pathFile = pathFile;
  }
  if (allowInvalidConstructors) {
    config.decaffeinateArgs = [].concat(toConsumableArray(config.decaffeinateArgs || []), ['--allow-invalid-constructors']);
  }
  if (landBase) {
    config.landBase = landBase;
  }
  if (decaffeinatePath) {
    config.decaffeinatePath = decaffeinatePath;
  }
  if (jscodeshiftPath) {
    config.jscodeshiftPath = jscodeshiftPath;
  }
  if (eslintPath) {
    config.eslintPath = eslintPath;
  }
  return config;
}

function resolveFilesToProcess(config, requireValidFiles) {
  var filesToProcess, pathFile, searchDirectory;
  return regeneratorRuntime.async(function resolveFilesToProcess$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          filesToProcess = config.filesToProcess, pathFile = config.pathFile, searchDirectory = config.searchDirectory;

          if (!filesToProcess) {
            _context3.next = 3;
            break;
          }

          return _context3.abrupt('return', filesToProcess);

        case 3:
          if (!pathFile) {
            _context3.next = 7;
            break;
          }

          _context3.next = 6;
          return regeneratorRuntime.awrap(getCoffeeFilesFromPathFile$1(pathFile, requireValidFiles));

        case 6:
          return _context3.abrupt('return', _context3.sent);

        case 7:
          if (!searchDirectory) {
            _context3.next = 11;
            break;
          }

          _context3.next = 10;
          return regeneratorRuntime.awrap(getCoffeeFilesUnderPath$1(searchDirectory));

        case 10:
          return _context3.abrupt('return', _context3.sent);

        case 11:
          _context3.next = 13;
          return regeneratorRuntime.awrap(getCoffeeFilesUnderPath$1('.'));

        case 13:
          return _context3.abrupt('return', _context3.sent);

        case 14:
        case 'end':
          return _context3.stop();
      }
    }
  }, null, this);
}

function resolveFileFilter(filesToProcess, config) {
  if (!config.fileFilterFn) {
    return filesToProcess;
  }
  return filesToProcess.filter(function (path$$1) {
    return config.fileFilterFn(resolve(path$$1));
  });
}

function resolveDecaffeinatePath(config) {
  return regeneratorRuntime.async(function resolveDecaffeinatePath$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          if (!config.decaffeinatePath) {
            _context4.next = 2;
            break;
          }

          return _context4.abrupt('return', config.decaffeinatePath);

        case 2:
          _context4.next = 4;
          return regeneratorRuntime.awrap(resolveBinary('decaffeinate'));

        case 4:
          return _context4.abrupt('return', _context4.sent);

        case 5:
        case 'end':
          return _context4.stop();
      }
    }
  }, null, this);
}

function resolveJscodeshiftPath(config) {
  return regeneratorRuntime.async(function resolveJscodeshiftPath$(_context5) {
    while (1) {
      switch (_context5.prev = _context5.next) {
        case 0:
          if (!(!config.jscodeshiftScripts && !config.fixImportsConfig)) {
            _context5.next = 2;
            break;
          }

          return _context5.abrupt('return', null);

        case 2:
          if (!config.jscodeshiftPath) {
            _context5.next = 4;
            break;
          }

          return _context5.abrupt('return', config.jscodeshiftPath);

        case 4:
          _context5.next = 6;
          return regeneratorRuntime.awrap(resolveBinary('jscodeshift'));

        case 6:
          return _context5.abrupt('return', _context5.sent);

        case 7:
        case 'end':
          return _context5.stop();
      }
    }
  }, null, this);
}

function resolveEslintPath(config) {
  return regeneratorRuntime.async(function resolveEslintPath$(_context6) {
    while (1) {
      switch (_context6.prev = _context6.next) {
        case 0:
          if (!config.eslintPath) {
            _context6.next = 2;
            break;
          }

          return _context6.abrupt('return', config.eslintPath);

        case 2:
          _context6.next = 4;
          return regeneratorRuntime.awrap(resolveBinary('eslint'));

        case 4:
          return _context6.abrupt('return', _context6.sent);

        case 5:
        case 'end':
          return _context6.stop();
      }
    }
  }, null, this);
}

/**
 * Determine the shell command that can be used to run the given binary,
 * prompting to globally install it if necessary.
 */
function resolveBinary(binaryName) {
  var nodeModulesPath, rl, answer;
  return regeneratorRuntime.async(function resolveBinary$(_context7) {
    while (1) {
      switch (_context7.prev = _context7.next) {
        case 0:
          nodeModulesPath = './node_modules/.bin/' + binaryName;
          _context7.next = 3;
          return regeneratorRuntime.awrap(exists(nodeModulesPath));

        case 3:
          if (!_context7.sent) {
            _context7.next = 7;
            break;
          }

          return _context7.abrupt('return', nodeModulesPath);

        case 7:
          _context7.prev = 7;
          _context7.next = 10;
          return regeneratorRuntime.awrap(exec('which ' + binaryName));

        case 10:
          return _context7.abrupt('return', binaryName);

        case 13:
          _context7.prev = 13;
          _context7.t0 = _context7['catch'](7);

          console.log(binaryName + ' binary not found on the PATH or in node_modules.');
          rl = readline.createInterface(process.stdin, process.stdout);
          _context7.next = 19;
          return regeneratorRuntime.awrap(rl.question('Run "npm install -g ' + binaryName + '"? [Y/n] '));

        case 19:
          answer = _context7.sent;

          rl.close();

          if (!answer.toLowerCase().startsWith('n')) {
            _context7.next = 23;
            break;
          }

          throw new CLIError(binaryName + ' must be installed.');

        case 23:
          console.log('Installing ' + binaryName + ' globally...');
          _context7.next = 26;
          return regeneratorRuntime.awrap(execLive('npm install -g ' + binaryName));

        case 26:
          console.log('Successfully installed ' + binaryName + '\n');
          return _context7.abrupt('return', binaryName);

        case 28:
        case 'end':
          return _context7.stop();
      }
    }
  }, null, this, [[7, 13]]);
}

function validateFilesToProcess(filesToProcess) {
  var _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, file, jsFile;

  return regeneratorRuntime.async(function validateFilesToProcess$(_context8) {
    while (1) {
      switch (_context8.prev = _context8.next) {
        case 0:
          _iteratorNormalCompletion2 = true;
          _didIteratorError2 = false;
          _iteratorError2 = undefined;
          _context8.prev = 3;
          _iterator2 = filesToProcess[Symbol.iterator]();

        case 5:
          if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
            _context8.next = 17;
            break;
          }

          file = _step2.value;

          if (file.endsWith('.coffee')) {
            _context8.next = 9;
            break;
          }

          throw new CLIError('The file ' + file + ' did not end with .coffee.');

        case 9:
          jsFile = file.substring(0, file.length - '.coffee'.length) + '.js';
          _context8.next = 12;
          return regeneratorRuntime.awrap(exists(jsFile));

        case 12:
          if (!_context8.sent) {
            _context8.next = 14;
            break;
          }

          throw new CLIError('The file ' + jsFile + ' already exists.');

        case 14:
          _iteratorNormalCompletion2 = true;
          _context8.next = 5;
          break;

        case 17:
          _context8.next = 23;
          break;

        case 19:
          _context8.prev = 19;
          _context8.t0 = _context8['catch'](3);
          _didIteratorError2 = true;
          _iteratorError2 = _context8.t0;

        case 23:
          _context8.prev = 23;
          _context8.prev = 24;

          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }

        case 26:
          _context8.prev = 26;

          if (!_didIteratorError2) {
            _context8.next = 29;
            break;
          }

          throw _iteratorError2;

        case 29:
          return _context8.finish(26);

        case 30:
          return _context8.finish(23);

        case 31:
        case 'end':
          return _context8.stop();
      }
    }
  }, null, this, [[3, 19, 23, 31], [24,, 26, 30]]);
}

/**
 * Determine if there are any uncommitted changes in the git state.
 */
var isWorktreeEmpty$1 = (function isWorktreeEmpty() {
  var status;
  return regeneratorRuntime.async(function isWorktreeEmpty$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return regeneratorRuntime.awrap(git().status());

        case 2:
          status = _context.sent;
          return _context.abrupt('return', status.files.length === 0);

        case 4:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this);
});

/**
 * Make an autogenerated commit with the "decaffeinate" author.
 */
var makeCommit$1 = (function makeCommit(commitMessage) {
  var userEmail, author;
  return regeneratorRuntime.async(function makeCommit$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return regeneratorRuntime.awrap(git().raw(['config', 'user.email']));

        case 2:
          userEmail = _context.sent;
          author = 'decaffeinate <' + userEmail + '>';
          _context.next = 6;
          return regeneratorRuntime.awrap(git().commit(commitMessage, { '--author': author, '--no-verify': null }));

        case 6:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this);
});

var convert$1 = (function convert(config) {
  var coffeeFiles, baseFiles, _config$decaffeinateA, decaffeinateArgs, decaffeinatePath, decaffeinateResults, runAsync, shortDescription, renameCommitMsg, decaffeinateCommitMsg, jsFiles, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, scriptPath, resolvedPath, regex, testFiles, thirdCommitModifiedFiles, _config$fixImportsCon, searchPath, absoluteImportPaths, _scriptPath, options, eligibleFixImportsFiles, eligibleRelativePaths, encodedOptions, eslintResults, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, result, _iteratorNormalCompletion3, _didIteratorError3, _iteratorError3, _iterator3, _step3, message, postProcessCommitMsg;

  return regeneratorRuntime.async(function convert$(_context9) {
    while (1) {
      switch (_context9.prev = _context9.next) {
        case 0:
          runAsync = function runAsync(description, asyncFn) {
            return regeneratorRuntime.async(function runAsync$(_context2) {
              while (1) {
                switch (_context2.prev = _context2.next) {
                  case 0:
                    _context2.next = 2;
                    return regeneratorRuntime.awrap(runWithProgressBar$1(description, baseFiles, function _callee(path$$1) {
                      return regeneratorRuntime.async(function _callee$(_context) {
                        while (1) {
                          switch (_context.prev = _context.next) {
                            case 0:
                              _context.next = 2;
                              return regeneratorRuntime.awrap(asyncFn(path$$1));

                            case 2:
                              return _context.abrupt('return', { path: path$$1 });

                            case 3:
                            case 'end':
                              return _context.stop();
                          }
                        }
                      }, null, this);
                    }));

                  case 2:
                  case 'end':
                    return _context2.stop();
                }
              }
            }, null, this);
          };

          _context9.next = 3;
          return regeneratorRuntime.awrap(assertGitWorktreeClean());

        case 3:
          coffeeFiles = config.filesToProcess;
          baseFiles = getBaseFiles(coffeeFiles);
          _config$decaffeinateA = config.decaffeinateArgs, decaffeinateArgs = _config$decaffeinateA === undefined ? [] : _config$decaffeinateA, decaffeinatePath = config.decaffeinatePath;

          if (config.skipVerify) {
            _context9.next = 12;
            break;
          }

          _context9.next = 9;
          return regeneratorRuntime.awrap(runWithProgressBar$1('Verifying that decaffeinate can successfully convert these files...', coffeeFiles, makeCLIFn(function (path$$1) {
            return decaffeinatePath + ' ' + decaffeinateArgs.join(' ') + ' < ' + path$$1;
          })));

        case 9:
          decaffeinateResults = _context9.sent;

          if (!(decaffeinateResults.filter(function (r) {
            return r.error !== null;
          }).length > 0)) {
            _context9.next = 12;
            break;
          }

          throw new CLIError('Some files could not be convered with decaffeinate.\nRe-run with the "check" command for more details.');

        case 12:
          _context9.next = 14;
          return regeneratorRuntime.awrap(runAsync('Backing up files to .original.coffee...', function _callee2(basePath) {
            return regeneratorRuntime.async(function _callee2$(_context3) {
              while (1) {
                switch (_context3.prev = _context3.next) {
                  case 0:
                    _context3.next = 2;
                    return regeneratorRuntime.awrap(copy(basePath + '.coffee', basePath + '.original.coffee'));

                  case 2:
                  case 'end':
                    return _context3.stop();
                }
              }
            }, null, this);
          }));

        case 14:
          _context9.next = 16;
          return regeneratorRuntime.awrap(runAsync('Renaming files from .coffee to .js...', function _callee3(basePath) {
            return regeneratorRuntime.async(function _callee3$(_context4) {
              while (1) {
                switch (_context4.prev = _context4.next) {
                  case 0:
                    _context4.next = 2;
                    return regeneratorRuntime.awrap(move(basePath + '.coffee', basePath + '.js'));

                  case 2:
                  case 'end':
                    return _context4.stop();
                }
              }
            }, null, this);
          }));

        case 16:
          shortDescription = getShortDescription(baseFiles);
          renameCommitMsg = 'decaffeinate: Rename ' + shortDescription + ' from .coffee to .js';

          console.log('Generating the first commit: "' + renameCommitMsg + '"...');
          _context9.next = 21;
          return regeneratorRuntime.awrap(git().rm(baseFiles.map(function (p) {
            return p + '.coffee';
          })));

        case 21:
          _context9.next = 23;
          return regeneratorRuntime.awrap(git().add(baseFiles.map(function (p) {
            return p + '.js';
          })));

        case 23:
          _context9.next = 25;
          return regeneratorRuntime.awrap(makeCommit$1(renameCommitMsg));

        case 25:
          _context9.next = 27;
          return regeneratorRuntime.awrap(runAsync('Moving files back...', function _callee4(basePath) {
            return regeneratorRuntime.async(function _callee4$(_context5) {
              while (1) {
                switch (_context5.prev = _context5.next) {
                  case 0:
                    _context5.next = 2;
                    return regeneratorRuntime.awrap(move(basePath + '.js', basePath + '.coffee'));

                  case 2:
                  case 'end':
                    return _context5.stop();
                }
              }
            }, null, this);
          }));

        case 27:
          _context9.next = 29;
          return regeneratorRuntime.awrap(runWithProgressBar$1('Running decaffeinate on all files...', coffeeFiles, makeCLIFn(function (path$$1) {
            return decaffeinatePath + ' ' + decaffeinateArgs.join(' ') + ' ' + path$$1;
          })));

        case 29:
          _context9.next = 31;
          return regeneratorRuntime.awrap(runAsync('Deleting old files...', function _callee5(basePath) {
            return regeneratorRuntime.async(function _callee5$(_context6) {
              while (1) {
                switch (_context6.prev = _context6.next) {
                  case 0:
                    _context6.next = 2;
                    return regeneratorRuntime.awrap(unlink$1(basePath + '.coffee'));

                  case 2:
                  case 'end':
                    return _context6.stop();
                }
              }
            }, null, this);
          }));

        case 31:
          decaffeinateCommitMsg = 'decaffeinate: Convert ' + shortDescription + ' to JS';

          console.log('Generating the second commit: ' + decaffeinateCommitMsg + '...');
          jsFiles = baseFiles.map(function (f) {
            return f + '.js';
          });
          _context9.next = 36;
          return regeneratorRuntime.awrap(git().add(jsFiles));

        case 36:
          _context9.next = 38;
          return regeneratorRuntime.awrap(makeCommit$1(decaffeinateCommitMsg));

        case 38:
          if (!config.jscodeshiftScripts) {
            _context9.next = 67;
            break;
          }

          _iteratorNormalCompletion = true;
          _didIteratorError = false;
          _iteratorError = undefined;
          _context9.prev = 42;
          _iterator = config.jscodeshiftScripts[Symbol.iterator]();

        case 44:
          if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
            _context9.next = 53;
            break;
          }

          scriptPath = _step.value;
          resolvedPath = resolveJscodeshiftScriptPath(scriptPath);

          console.log('Running jscodeshift script ' + resolvedPath + '...');
          _context9.next = 50;
          return regeneratorRuntime.awrap(execLive(config.jscodeshiftPath + ' --parser flow         -t ' + resolvedPath + ' ' + jsFiles.join(' ')));

        case 50:
          _iteratorNormalCompletion = true;
          _context9.next = 44;
          break;

        case 53:
          _context9.next = 59;
          break;

        case 55:
          _context9.prev = 55;
          _context9.t0 = _context9['catch'](42);
          _didIteratorError = true;
          _iteratorError = _context9.t0;

        case 59:
          _context9.prev = 59;
          _context9.prev = 60;

          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }

        case 62:
          _context9.prev = 62;

          if (!_didIteratorError) {
            _context9.next = 65;
            break;
          }

          throw _iteratorError;

        case 65:
          return _context9.finish(62);

        case 66:
          return _context9.finish(59);

        case 67:
          if (!config.mochaEnvFilePattern) {
            _context9.next = 72;
            break;
          }

          regex = new RegExp(config.mochaEnvFilePattern);
          testFiles = jsFiles.filter(function (f) {
            return regex.test(f);
          });
          _context9.next = 72;
          return regeneratorRuntime.awrap(runWithProgressBar$1('Adding /* eslint-env mocha */ to test files...', testFiles, function _callee6(path$$1) {
            return regeneratorRuntime.async(function _callee6$(_context7) {
              while (1) {
                switch (_context7.prev = _context7.next) {
                  case 0:
                    _context7.next = 2;
                    return regeneratorRuntime.awrap(prependToFile(path$$1, '/* eslint-env mocha */\n'));

                  case 2:
                    return _context7.abrupt('return', { error: null });

                  case 3:
                  case 'end':
                    return _context7.stop();
                }
              }
            }, null, this);
          }));

        case 72:
          thirdCommitModifiedFiles = jsFiles.slice();

          if (!config.fixImportsConfig) {
            _context9.next = 88;
            break;
          }

          _config$fixImportsCon = config.fixImportsConfig, searchPath = _config$fixImportsCon.searchPath, absoluteImportPaths = _config$fixImportsCon.absoluteImportPaths;

          if (!absoluteImportPaths) {
            absoluteImportPaths = [];
          }
          _scriptPath = path.join(__dirname, '../jscodeshift-scripts-dist/fix-imports.js');
          options = {
            convertedFiles: jsFiles.map(function (p) {
              return path.resolve(p);
            }),
            absoluteImportPaths: absoluteImportPaths.map(function (p) {
              return path.resolve(p);
            })
          };
          _context9.next = 80;
          return regeneratorRuntime.awrap(getEligibleFixImportsFiles(searchPath, jsFiles));

        case 80:
          eligibleFixImportsFiles = _context9.sent;

          console.log('Fixing any imports across the whole codebase...');

          if (!(eligibleFixImportsFiles.length > 0)) {
            _context9.next = 88;
            break;
          }

          // Note that the args can get really long, so we take reasonable steps to
          // reduce the chance of hitting the system limit on arg length
          // (256K by default on Mac).
          eligibleRelativePaths = eligibleFixImportsFiles.map(function (p) {
            return path.relative('', p);
          });

          thirdCommitModifiedFiles = eligibleFixImportsFiles;
          encodedOptions = zlib.deflateSync(JSON.stringify(options)).toString('base64');
          _context9.next = 88;
          return regeneratorRuntime.awrap(execLive('      ' + config.jscodeshiftPath + ' --parser flow -t ' + _scriptPath + '         ' + eligibleRelativePaths.join(' ') + ' --encoded-options=' + encodedOptions));

        case 88:
          _context9.next = 90;
          return regeneratorRuntime.awrap(runWithProgressBar$1('Running eslint --fix on all files...', baseFiles, makeEslintFixFn(config)));

        case 90:
          eslintResults = _context9.sent;
          _iteratorNormalCompletion2 = true;
          _didIteratorError2 = false;
          _iteratorError2 = undefined;
          _context9.prev = 94;
          _iterator2 = eslintResults[Symbol.iterator]();

        case 96:
          if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
            _context9.next = 120;
            break;
          }

          result = _step2.value;
          _iteratorNormalCompletion3 = true;
          _didIteratorError3 = false;
          _iteratorError3 = undefined;
          _context9.prev = 101;

          for (_iterator3 = result.messages[Symbol.iterator](); !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            message = _step3.value;

            console.log(message);
          }
          _context9.next = 109;
          break;

        case 105:
          _context9.prev = 105;
          _context9.t1 = _context9['catch'](101);
          _didIteratorError3 = true;
          _iteratorError3 = _context9.t1;

        case 109:
          _context9.prev = 109;
          _context9.prev = 110;

          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }

        case 112:
          _context9.prev = 112;

          if (!_didIteratorError3) {
            _context9.next = 115;
            break;
          }

          throw _iteratorError3;

        case 115:
          return _context9.finish(112);

        case 116:
          return _context9.finish(109);

        case 117:
          _iteratorNormalCompletion2 = true;
          _context9.next = 96;
          break;

        case 120:
          _context9.next = 126;
          break;

        case 122:
          _context9.prev = 122;
          _context9.t2 = _context9['catch'](94);
          _didIteratorError2 = true;
          _iteratorError2 = _context9.t2;

        case 126:
          _context9.prev = 126;
          _context9.prev = 127;

          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }

        case 129:
          _context9.prev = 129;

          if (!_didIteratorError2) {
            _context9.next = 132;
            break;
          }

          throw _iteratorError2;

        case 132:
          return _context9.finish(129);

        case 133:
          return _context9.finish(126);

        case 134:
          if (!config.codePrefix) {
            _context9.next = 137;
            break;
          }

          _context9.next = 137;
          return regeneratorRuntime.awrap(runWithProgressBar$1('Adding code prefix to converted files...', baseFiles, function _callee7(path$$1) {
            return regeneratorRuntime.async(function _callee7$(_context8) {
              while (1) {
                switch (_context8.prev = _context8.next) {
                  case 0:
                    _context8.next = 2;
                    return regeneratorRuntime.awrap(prependToFile(path$$1 + '.js', config.codePrefix));

                  case 2:
                    return _context8.abrupt('return', { error: null });

                  case 3:
                  case 'end':
                    return _context8.stop();
                }
              }
            }, null, this);
          }));

        case 137:
          postProcessCommitMsg = 'decaffeinate: Run post-processing cleanups on ' + shortDescription;

          console.log('Generating the third commit: ' + postProcessCommitMsg + '...');
          _context9.next = 141;
          return regeneratorRuntime.awrap(git().add(thirdCommitModifiedFiles));

        case 141:
          _context9.next = 143;
          return regeneratorRuntime.awrap(makeCommit$1(postProcessCommitMsg));

        case 143:

          console.log('Successfully ran decaffeinate on ' + pluralize(baseFiles.length, 'file') + '.');
          console.log('You should now fix lint issues in any affected files.');
          console.log('All CoffeeScript files were backed up as .original.coffee files that you can use for comparison.');
          console.log('You can run "bulk-decaffeinate clean" to remove those files.');
          console.log('To allow git to properly track file history, you should NOT squash the generated commits together.');

        case 148:
        case 'end':
          return _context9.stop();
      }
    }
  }, null, this, [[42, 55, 59, 67], [60,, 62, 66], [94, 122, 126, 134], [101, 105, 109, 117], [110,, 112, 116], [127,, 129, 133]]);
});

function assertGitWorktreeClean() {
  return regeneratorRuntime.async(function assertGitWorktreeClean$(_context10) {
    while (1) {
      switch (_context10.prev = _context10.next) {
        case 0:
          _context10.next = 2;
          return regeneratorRuntime.awrap(isWorktreeEmpty$1());

        case 2:
          if (_context10.sent) {
            _context10.next = 4;
            break;
          }

          throw new CLIError('You have modifications to your git worktree.\nPlease revert or commit them before running convert.');

        case 4:
        case 'end':
          return _context10.stop();
      }
    }
  }, null, this);
}

function getBaseFiles(coffeeFiles) {
  return coffeeFiles.map(function (coffeeFile) {
    if (!coffeeFile.endsWith('.coffee')) {
      throw new CLIError('The non-CoffeeScript file ' + coffeeFile + ' was specified.');
    }
    return coffeeFile.substring(0, coffeeFile.length - '.coffee'.length);
  });
}

function getShortDescription(baseFiles) {
  var firstFile = path.basename(baseFiles[0]) + '.coffee';
  if (baseFiles.length === 1) {
    return firstFile;
  } else {
    return firstFile + ' and ' + pluralize(baseFiles.length - 1, 'other file');
  }
}

function resolveJscodeshiftScriptPath(scriptPath) {
  if (['prefer-function-declarations.js', 'remove-coffee-from-imports.js', 'top-level-this-to-exports.js'].includes(scriptPath)) {
    return path.join(__dirname, '../jscodeshift-scripts-dist/' + scriptPath);
  }
  return scriptPath;
}

function getEligibleFixImportsFiles(searchPath, jsFiles) {
  var jsBasenames, resolvedPaths, allJsFiles;
  return regeneratorRuntime.async(function getEligibleFixImportsFiles$(_context12) {
    while (1) {
      switch (_context12.prev = _context12.next) {
        case 0:
          jsBasenames = jsFiles.map(function (p) {
            return path.basename(p, '.js');
          });
          resolvedPaths = jsFiles.map(function (p) {
            return path.resolve(p);
          });
          _context12.next = 4;
          return regeneratorRuntime.awrap(getFilesUnderPath$1(searchPath, function (p) {
            return p.endsWith('.js');
          }));

        case 4:
          allJsFiles = _context12.sent;
          _context12.next = 7;
          return regeneratorRuntime.awrap(runWithProgressBar$1('Searching for files that may need to have updated imports...', allJsFiles, function _callee8(p) {
            var resolvedPath, contents, _iteratorNormalCompletion4, _didIteratorError4, _iteratorError4, _iterator4, _step4, basename;

            return regeneratorRuntime.async(function _callee8$(_context11) {
              while (1) {
                switch (_context11.prev = _context11.next) {
                  case 0:
                    resolvedPath = path.resolve(p);

                    if (!resolvedPaths.includes(resolvedPath)) {
                      _context11.next = 3;
                      break;
                    }

                    return _context11.abrupt('return', { error: null });

                  case 3:
                    _context11.next = 5;
                    return regeneratorRuntime.awrap(readFile$1(resolvedPath));

                  case 5:
                    contents = _context11.sent.toString();
                    _iteratorNormalCompletion4 = true;
                    _didIteratorError4 = false;
                    _iteratorError4 = undefined;
                    _context11.prev = 9;
                    _iterator4 = jsBasenames[Symbol.iterator]();

                  case 11:
                    if (_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done) {
                      _context11.next = 19;
                      break;
                    }

                    basename = _step4.value;

                    if (!contents.includes(basename)) {
                      _context11.next = 16;
                      break;
                    }

                    resolvedPaths.push(resolvedPath);
                    return _context11.abrupt('return', { error: null });

                  case 16:
                    _iteratorNormalCompletion4 = true;
                    _context11.next = 11;
                    break;

                  case 19:
                    _context11.next = 25;
                    break;

                  case 21:
                    _context11.prev = 21;
                    _context11.t0 = _context11['catch'](9);
                    _didIteratorError4 = true;
                    _iteratorError4 = _context11.t0;

                  case 25:
                    _context11.prev = 25;
                    _context11.prev = 26;

                    if (!_iteratorNormalCompletion4 && _iterator4.return) {
                      _iterator4.return();
                    }

                  case 28:
                    _context11.prev = 28;

                    if (!_didIteratorError4) {
                      _context11.next = 31;
                      break;
                    }

                    throw _iteratorError4;

                  case 31:
                    return _context11.finish(28);

                  case 32:
                    return _context11.finish(25);

                  case 33:
                    return _context11.abrupt('return', { error: null });

                  case 34:
                  case 'end':
                    return _context11.stop();
                }
              }
            }, null, this, [[9, 21, 25, 33], [26,, 28, 32]]);
          }));

        case 7:
          return _context12.abrupt('return', resolvedPaths);

        case 8:
        case 'end':
          return _context12.stop();
      }
    }
  }, null, this);
}

function makeEslintFixFn(config) {
  return function runEslint(path$$1) {
    var messages, eslintOutputStr, ruleIds, eslintOutput, suggestionLine;
    return regeneratorRuntime.async(function runEslint$(_context13) {
      while (1) {
        switch (_context13.prev = _context13.next) {
          case 0:
            messages = [];

            // Ignore the eslint exit code; it gives useful stdout in the same format
            // regardless of the exit code. Also keep a 10MB buffer since sometimes
            // there can be a LOT of lint failures.

            _context13.next = 3;
            return regeneratorRuntime.awrap(exec(config.eslintPath + ' --fix --format json ' + path$$1 + '.js; :', { maxBuffer: 10000 * 1024 }));

          case 3:
            eslintOutputStr = _context13.sent[0];
            ruleIds = void 0;

            if (eslintOutputStr.includes("ESLint couldn't find a configuration file")) {
              messages.push('Skipping "eslint --fix" on ' + path$$1 + '.js because there was no eslint config file.');
              ruleIds = [];
            } else {
              eslintOutput = JSON.parse(eslintOutputStr);

              ruleIds = eslintOutput[0].messages.map(function (message) {
                return message.ruleId;
              }).filter(function (ruleId) {
                return ruleId;
              });
              ruleIds = Array.from(new Set(ruleIds)).sort();
            }

            suggestionLine = void 0;

            if (ruleIds.length > 0) {
              suggestionLine = 'Fix any style issues and re-enable lint.';
            } else {
              suggestionLine = 'Sanity-check the conversion and remove this comment.';
            }

            _context13.next = 10;
            return regeneratorRuntime.awrap(prependToFile(path$$1 + '.js', '// TODO: This file was created by bulk-decaffeinate.\n// ' + suggestionLine + '\n'));

          case 10:
            if (!(ruleIds.length > 0)) {
              _context13.next = 13;
              break;
            }

            _context13.next = 13;
            return regeneratorRuntime.awrap(prependToFile(path$$1 + '.js', '/* eslint-disable\n' + ruleIds.map(function (ruleId) {
              return '    ' + ruleId + ',';
            }).join('\n') + '\n*/\n'));

          case 13:
            return _context13.abrupt('return', { error: null, messages: messages });

          case 14:
          case 'end':
            return _context13.stop();
        }
      }
    }, null, this);
  };
}

function prependToFile(path$$1, prependText) {
  var contents;
  return regeneratorRuntime.async(function prependToFile$(_context14) {
    while (1) {
      switch (_context14.prev = _context14.next) {
        case 0:
          _context14.next = 2;
          return regeneratorRuntime.awrap(readFile$1(path$$1));

        case 2:
          contents = _context14.sent;

          contents = prependText + contents;
          _context14.next = 6;
          return regeneratorRuntime.awrap(writeFile$1(path$$1, contents));

        case 6:
        case 'end':
          return _context14.stop();
      }
    }
  }, null, this);
}

/**
 * The land option "packages" a set of commits into a single merge commit that
 * can be pushed. Splitting the decaffeinate work up into different commits
 * allows git to properly track file history when a file is changed from
 * CoffeeScript to JavaScript.
 *
 * A typical use case is that the merge commit will include 4 commits: the three
 * auto-generated decaffeinate commits and a follow-up commit to fix lint
 * errors. Unlike the auto-generated decaffeinate commits, the merge commit is
 * created with the default author name.
 */
var land$1 = (function land(config) {
  var remote, upstreamBranch, phabricatorAware, remoteBranch, commits, differentialRevisionLine, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, commit, status, message, cherryPickHeadCommit, mergeMessage;

  return regeneratorRuntime.async(function land$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          remote = config.landConfig && config.landConfig.remote;
          upstreamBranch = config.landConfig && config.landConfig.upstreamBranch;
          phabricatorAware = config.landConfig && config.landConfig.phabricatorAware;

          if (!remote) {
            console.log('No remote was specified. Defaulting to origin.');
            remote = 'origin';
          }
          if (!upstreamBranch) {
            console.log('No upstreamBranch was specified. Defaulting to master.');
            upstreamBranch = 'master';
          }
          remoteBranch = remote + '/' + upstreamBranch;

          console.log('Running fetch for ' + remote + '.');
          _context.next = 9;
          return regeneratorRuntime.awrap(git().fetch([remote]));

        case 9:
          _context.next = 11;
          return regeneratorRuntime.awrap(getCommits(config));

        case 11:
          commits = _context.sent;

          console.log('Found ' + commits.length + ' commits to use.');

          if (!phabricatorAware) {
            _context.next = 19;
            break;
          }

          _context.next = 16;
          return regeneratorRuntime.awrap(getDifferentialRevisionLine(commits));

        case 16:
          _context.t0 = _context.sent;
          _context.next = 20;
          break;

        case 19:
          _context.t0 = null;

        case 20:
          differentialRevisionLine = _context.t0;
          _context.next = 23;
          return regeneratorRuntime.awrap(git().checkout(remoteBranch));

        case 23:
          _iteratorNormalCompletion = true;
          _didIteratorError = false;
          _iteratorError = undefined;
          _context.prev = 26;
          _iterator = commits[Symbol.iterator]();

        case 28:
          if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
            _context.next = 49;
            break;
          }

          commit = _step.value;

          console.log('Cherry-picking "' + commit.message + '"');
          _context.next = 33;
          return regeneratorRuntime.awrap(git().raw(['cherry-pick', commit.hash]));

        case 33:
          _context.next = 35;
          return regeneratorRuntime.awrap(git().status());

        case 35:
          status = _context.sent;

          if (!(status.conflicted.length > 0)) {
            _context.next = 38;
            break;
          }

          throw new CLIError('The cherry pick had conflicts.\nPlease rebase your changes and retry "bulk-decaffeinate land"');

        case 38:
          _context.next = 40;
          return regeneratorRuntime.awrap(getCommitMessage(commit.hash));

        case 40:
          message = _context.sent;

          if (!phabricatorAware) {
            _context.next = 46;
            break;
          }

          if (message.includes('Differential Revision')) {
            _context.next = 46;
            break;
          }

          message += '\n\n' + differentialRevisionLine;
          _context.next = 46;
          return regeneratorRuntime.awrap(git().commit(message, ['--amend']));

        case 46:
          _iteratorNormalCompletion = true;
          _context.next = 28;
          break;

        case 49:
          _context.next = 55;
          break;

        case 51:
          _context.prev = 51;
          _context.t1 = _context['catch'](26);
          _didIteratorError = true;
          _iteratorError = _context.t1;

        case 55:
          _context.prev = 55;
          _context.prev = 56;

          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }

        case 58:
          _context.prev = 58;

          if (!_didIteratorError) {
            _context.next = 61;
            break;
          }

          throw _iteratorError;

        case 61:
          return _context.finish(58);

        case 62:
          return _context.finish(55);

        case 63:

          console.log('Creating merge commit on ' + remoteBranch);
          _context.next = 66;
          return regeneratorRuntime.awrap(git().revparse(['HEAD']));

        case 66:
          cherryPickHeadCommit = _context.sent.trim();
          _context.next = 69;
          return regeneratorRuntime.awrap(git().checkout(remoteBranch));

        case 69:
          mergeMessage = 'Merge decaffeinate changes into ' + remoteBranch;

          if (phabricatorAware) {
            mergeMessage += '\n\n' + differentialRevisionLine;
          }
          _context.next = 73;
          return regeneratorRuntime.awrap(git().mergeFromTo(cherryPickHeadCommit, 'HEAD', ['--no-ff']));

        case 73:
          _context.next = 75;
          return regeneratorRuntime.awrap(git().commit(mergeMessage, ['--amend']));

        case 75:
          if (!phabricatorAware) {
            _context.next = 79;
            break;
          }

          console.log('Pulling commit message from Phabricator.');
          _context.next = 79;
          return regeneratorRuntime.awrap(exec('arc amend'));

        case 79:
          console.log('');
          console.log('Done. Please verify that the git history looks right.');
          console.log('You can push your changes with a command like this:');
          console.log('git push ' + remote + ' HEAD:' + upstreamBranch);
          console.log('If you get a conflict, you should re-run "bulk-decaffeinate land".');

        case 84:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this, [[26, 51, 55, 63], [56,, 58, 62]]);
});

function getCommits(config) {
  var explicitBase, allCommits, commits, hasSeenDecaffeinateCommit, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, commit, isDecaffeinateCommit;

  return regeneratorRuntime.async(function getCommits$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          explicitBase = null;

          if (!config.landBase) {
            _context2.next = 5;
            break;
          }

          _context2.next = 4;
          return regeneratorRuntime.awrap(git().revparse([config.landBase]));

        case 4:
          explicitBase = _context2.sent.trim();

        case 5:
          allCommits = void 0;
          _context2.prev = 6;
          _context2.next = 9;
          return regeneratorRuntime.awrap(git().log({ from: 'HEAD', to: 'HEAD~20' }));

        case 9:
          allCommits = _context2.sent.all;
          _context2.next = 17;
          break;

        case 12:
          _context2.prev = 12;
          _context2.t0 = _context2['catch'](6);
          _context2.next = 16;
          return regeneratorRuntime.awrap(git().log({ from: 'HEAD' }));

        case 16:
          allCommits = _context2.sent.all;

        case 17:
          commits = [];
          hasSeenDecaffeinateCommit = false;
          _iteratorNormalCompletion2 = true;
          _didIteratorError2 = false;
          _iteratorError2 = undefined;
          _context2.prev = 22;
          _iterator2 = allCommits[Symbol.iterator]();

        case 24:
          if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
            _context2.next = 39;
            break;
          }

          commit = _step2.value;
          isDecaffeinateCommit = commit.author_name === 'decaffeinate';

          if (!(explicitBase !== null)) {
            _context2.next = 32;
            break;
          }

          if (!(explicitBase === commit.hash)) {
            _context2.next = 30;
            break;
          }

          return _context2.abrupt('return', commits);

        case 30:
          _context2.next = 34;
          break;

        case 32:
          if (!(hasSeenDecaffeinateCommit && !isDecaffeinateCommit)) {
            _context2.next = 34;
            break;
          }

          return _context2.abrupt('return', commits);

        case 34:
          if (!hasSeenDecaffeinateCommit && isDecaffeinateCommit) {
            hasSeenDecaffeinateCommit = true;
          }
          commits.unshift(commit);

        case 36:
          _iteratorNormalCompletion2 = true;
          _context2.next = 24;
          break;

        case 39:
          _context2.next = 45;
          break;

        case 41:
          _context2.prev = 41;
          _context2.t1 = _context2['catch'](22);
          _didIteratorError2 = true;
          _iteratorError2 = _context2.t1;

        case 45:
          _context2.prev = 45;
          _context2.prev = 46;

          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }

        case 48:
          _context2.prev = 48;

          if (!_didIteratorError2) {
            _context2.next = 51;
            break;
          }

          throw _iteratorError2;

        case 51:
          return _context2.finish(48);

        case 52:
          return _context2.finish(45);

        case 53:
          throw new CLIError('Searched 20 commits without finding a set of commits to use. Make sure you have\ncommits with the "decaffeinate" author in your recent git history, and that the\nfirst of those commits is the first commit that you would like to land.');

        case 54:
        case 'end':
          return _context2.stop();
      }
    }
  }, null, this, [[6, 12], [22, 41, 45, 53], [46,, 48, 52]]);
}

function getDifferentialRevisionLine(commits) {
  var resultLine, _iteratorNormalCompletion3, _didIteratorError3, _iteratorError3, _iterator3, _step3, commit, commitMessage, _iteratorNormalCompletion4, _didIteratorError4, _iteratorError4, _iterator4, _step4, line;

  return regeneratorRuntime.async(function getDifferentialRevisionLine$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          resultLine = null;
          _iteratorNormalCompletion3 = true;
          _didIteratorError3 = false;
          _iteratorError3 = undefined;
          _context3.prev = 4;
          _iterator3 = commits[Symbol.iterator]();

        case 6:
          if (_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done) {
            _context3.next = 44;
            break;
          }

          commit = _step3.value;
          _context3.next = 10;
          return regeneratorRuntime.awrap(getCommitMessage(commit.hash));

        case 10:
          commitMessage = _context3.sent;
          _iteratorNormalCompletion4 = true;
          _didIteratorError4 = false;
          _iteratorError4 = undefined;
          _context3.prev = 14;
          _iterator4 = commitMessage.split('\n')[Symbol.iterator]();

        case 16:
          if (_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done) {
            _context3.next = 27;
            break;
          }

          line = _step4.value;

          if (!line.startsWith('Differential Revision')) {
            _context3.next = 24;
            break;
          }

          if (!(resultLine === null || resultLine === line)) {
            _context3.next = 23;
            break;
          }

          resultLine = line;
          _context3.next = 24;
          break;

        case 23:
          throw new CLIError('Found multiple different "Differential Revision" lines in the matched commits.\nPlease set your git HEAD so that only one Phabricator code review is included.');

        case 24:
          _iteratorNormalCompletion4 = true;
          _context3.next = 16;
          break;

        case 27:
          _context3.next = 33;
          break;

        case 29:
          _context3.prev = 29;
          _context3.t0 = _context3['catch'](14);
          _didIteratorError4 = true;
          _iteratorError4 = _context3.t0;

        case 33:
          _context3.prev = 33;
          _context3.prev = 34;

          if (!_iteratorNormalCompletion4 && _iterator4.return) {
            _iterator4.return();
          }

        case 36:
          _context3.prev = 36;

          if (!_didIteratorError4) {
            _context3.next = 39;
            break;
          }

          throw _iteratorError4;

        case 39:
          return _context3.finish(36);

        case 40:
          return _context3.finish(33);

        case 41:
          _iteratorNormalCompletion3 = true;
          _context3.next = 6;
          break;

        case 44:
          _context3.next = 50;
          break;

        case 46:
          _context3.prev = 46;
          _context3.t1 = _context3['catch'](4);
          _didIteratorError3 = true;
          _iteratorError3 = _context3.t1;

        case 50:
          _context3.prev = 50;
          _context3.prev = 51;

          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }

        case 53:
          _context3.prev = 53;

          if (!_didIteratorError3) {
            _context3.next = 56;
            break;
          }

          throw _iteratorError3;

        case 56:
          return _context3.finish(53);

        case 57:
          return _context3.finish(50);

        case 58:
          if (!(resultLine === null)) {
            _context3.next = 60;
            break;
          }

          throw new CLIError('\nExpected to find a "Differential Revision" line in at least one commit.');

        case 60:
          return _context3.abrupt('return', resultLine);

        case 61:
        case 'end':
          return _context3.stop();
      }
    }
  }, null, this, [[4, 46, 50, 58], [14, 29, 33, 41], [34,, 36, 40], [51,, 53, 57]]);
}

function getCommitMessage(commitHash) {
  return regeneratorRuntime.async(function getCommitMessage$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          _context4.next = 2;
          return regeneratorRuntime.awrap(git().show(['-s', '--format=%B', commitHash]));

        case 2:
          return _context4.abrupt('return', _context4.sent);

        case 3:
        case 'end':
          return _context4.stop();
      }
    }
  }, null, this);
}

var viewErrors$1 = (function viewErrors() {
  var resultsJson, results, filesToOpen, rl, answer, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, path$$1;

  return regeneratorRuntime.async(function viewErrors$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return regeneratorRuntime.awrap(exists('decaffeinate-results.json'));

        case 2:
          if (_context.sent) {
            _context.next = 5;
            break;
          }

          console.log('decaffeinate-results.json file not found. Please run the "check" command first.');
          return _context.abrupt('return');

        case 5:
          _context.next = 7;
          return regeneratorRuntime.awrap(readFile('decaffeinate-results.json'));

        case 7:
          resultsJson = _context.sent;
          results = JSON.parse(resultsJson);
          filesToOpen = results.filter(function (r) {
            return r.error !== null;
          }).map(function (r) {
            return r.path;
          });

          if (!(filesToOpen.length === 0)) {
            _context.next = 13;
            break;
          }

          console.log('No failures were found!');
          return _context.abrupt('return');

        case 13:
          if (!(filesToOpen.length > 10)) {
            _context.next = 21;
            break;
          }

          rl = readline.createInterface(process.stdin, process.stdout);
          _context.next = 17;
          return regeneratorRuntime.awrap(rl.question('This will open ' + filesToOpen.length + ' browser tabs. Do you want to proceed? [y/N] '));

        case 17:
          answer = _context.sent;

          rl.close();

          if (answer.toLowerCase().startsWith('y')) {
            _context.next = 21;
            break;
          }

          return _context.abrupt('return');

        case 21:
          _iteratorNormalCompletion = true;
          _didIteratorError = false;
          _iteratorError = undefined;
          _context.prev = 24;
          _iterator = filesToOpen[Symbol.iterator]();

        case 26:
          if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
            _context.next = 33;
            break;
          }

          path$$1 = _step.value;
          _context.next = 30;
          return regeneratorRuntime.awrap(openInRepl(path$$1));

        case 30:
          _iteratorNormalCompletion = true;
          _context.next = 26;
          break;

        case 33:
          _context.next = 39;
          break;

        case 35:
          _context.prev = 35;
          _context.t0 = _context['catch'](24);
          _didIteratorError = true;
          _iteratorError = _context.t0;

        case 39:
          _context.prev = 39;
          _context.prev = 40;

          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }

        case 42:
          _context.prev = 42;

          if (!_didIteratorError) {
            _context.next = 45;
            break;
          }

          throw _iteratorError;

        case 45:
          return _context.finish(42);

        case 46:
          return _context.finish(39);

        case 47:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this, [[24, 35, 39, 47], [40,, 42, 46]]);
});

function openInRepl(path$$1) {
  var fileContents, encodedFile, url;
  return regeneratorRuntime.async(function openInRepl$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.next = 2;
          return regeneratorRuntime.awrap(readFile(path$$1));

        case 2:
          fileContents = _context2.sent;
          encodedFile = encodeURIComponent(fileContents);
          url = 'http://decaffeinate-project.org/repl/#?evaluate=false&stage=full&code=' + encodedFile;
          _context2.next = 7;
          return regeneratorRuntime.awrap(opn(url, { wait: false }));

        case 7:
        case 'end':
          return _context2.stop();
      }
    }
  }, null, this);
}

var cli = function () {
  var command = null;
  commander.arguments('<command>').description('Run decaffeinate on a set of files.\n\n  Commands:\n    check: Try decaffeinate on the specified files and generate a report of which files can be\n           converted. By default, all .coffee files in the current directory are used.\n    convert: Run decaffeinate on the specified files and generate git commits for the transition.\n    view-errors: Open failures from the most recent run in an online repl.\n    clean: Delete all files ending with .original.coffee in the current working directory or any\n           of its subdirectories.\n    land: Create a merge commit with all commits generated by bulk-decaffeinate.').action(function (commandArg) {
    return command = commandArg;
  }).option('-f, --file [path]', 'An absolute or relative path to decaffeinate. This arg may be\n                            specified multiple times.', function (arg, args) {
    args.push(arg);return args;
  }, []).option('-p, --path-file [path]', 'A file containing the paths of .coffee files to decaffeinate, one\n                            path per line. Paths can be either absolute or relative to the\n                            current working directory.').option('-d, --dir [path]', 'A directory containing files to decaffeinate. All .coffee files in\n                            any subdirectory of this directory are considered for decaffeinate.').option('--allow-invalid-constructors', 'If specified, the --allow-invalid-constructors arg is added when\n                            invoking decaffeinate.').option('--land-base [revision]', 'The git revision to use as the base commit when running the "land"\n                            command. If none is specified, bulk-decaffeinate tries to use the\n                            first auto-generated commit in recent history.').option('--decaffeinate-path [path]', 'The path to the decaffeinate binary. If none is specified, it will\n                            be automatically discovered from node_modules and then from the\n                            PATH.').option('--jscodeshift-path [path]', 'The path to the jscodeshift binary. If none is specified, it will be\n                            automatically discovered from node_modules and then from the PATH.').option('--eslint-path [path]', 'The path to the eslint binary. If none is specified, it will be\n                            automatically discovered from node_modules and then from the PATH.').parse(process.argv);

  runCommand(command);
};

function runCommand(command) {
  var config, _config, _config2;

  return regeneratorRuntime.async(function runCommand$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.prev = 0;

          if (!(command === 'check')) {
            _context.next = 9;
            break;
          }

          _context.next = 4;
          return regeneratorRuntime.awrap(resolveConfig$1(commander));

        case 4:
          config = _context.sent;
          _context.next = 7;
          return regeneratorRuntime.awrap(check$1(config));

        case 7:
          _context.next = 36;
          break;

        case 9:
          if (!(command === 'convert')) {
            _context.next = 17;
            break;
          }

          _context.next = 12;
          return regeneratorRuntime.awrap(resolveConfig$1(commander));

        case 12:
          _config = _context.sent;
          _context.next = 15;
          return regeneratorRuntime.awrap(convert$1(_config));

        case 15:
          _context.next = 36;
          break;

        case 17:
          if (!(command === 'view-errors')) {
            _context.next = 22;
            break;
          }

          _context.next = 20;
          return regeneratorRuntime.awrap(viewErrors$1());

        case 20:
          _context.next = 36;
          break;

        case 22:
          if (!(command === 'clean')) {
            _context.next = 27;
            break;
          }

          _context.next = 25;
          return regeneratorRuntime.awrap(clean$1());

        case 25:
          _context.next = 36;
          break;

        case 27:
          if (!(command === 'land')) {
            _context.next = 35;
            break;
          }

          _context.next = 30;
          return regeneratorRuntime.awrap(resolveConfig$1(commander, false));

        case 30:
          _config2 = _context.sent;
          _context.next = 33;
          return regeneratorRuntime.awrap(land$1(_config2));

        case 33:
          _context.next = 36;
          break;

        case 35:
          commander.outputHelp();

        case 36:
          _context.next = 42;
          break;

        case 38:
          _context.prev = 38;
          _context.t0 = _context['catch'](0);

          process.exitCode = 1;
          console.error(CLIError.formatError(_context.t0));

        case 42:
        case 'end':
          return _context.stop();
      }
    }
  }, null, this, [[0, 38]]);
}

export default cli;
//# sourceMappingURL=bulk-decaffeinate.mjs.map
